@@ -19,6 +19,16 @@ architecture=$(dpkg --print-architecture)
 
 repositories_dir=/host/repositories
 
+case "${os}-${code-name}" in
+  debian-bullseye)
+    :
+    ;;
+  debian-*)
+    wget https://apache.jfrog.io/artifactory/arrow/${os}/apache-arrow-apt-source-latest-${code_name}.deb
+    apt install -V -y ./apache-arrow-apt-source-latest-${code_name}.deb
+    ;;
+esac
+
 wget \
   https://packages.groonga.org/${os}/groonga-apt-source-latest-${code_name}.deb
 apt install -V -y ./groonga-apt-source-latest-${code_name}.deb
@@ -51,14 +61,13 @@ echo "::endgroup::"
 
 echo "::group::Install packages for test"
 
-postgresql_version=$(dpkg -l | \
-                       grep pgroonga | \
-                       grep -E -o '[0-9.]+' |
-                       head -n1)
-postgresql_package_prefix=$(dpkg -l | \
-                              grep pgroonga | \
-                              grep -E -o 'postgresql-[0-9.]+(-pgdg)?' |
-                              head -n1)
+pgroonga_package=$(dpkg -l | \
+                     grep pgroonga | \
+                     head -n1 | \
+                     awk '{print $2}')
+postgresql_version=$(echo ${pgroonga_package} | grep -E -o '[0-9.]+')
+postgresql_package_prefix=$(echo ${pgroonga_package} | \
+                              grep -E -o 'postgresql-[0-9.]+(-pgdg)?')
 if ! echo "${postgresql_package_prefix}" | grep -q pgdg; then
   apt install -V -y \
       $(echo ${postgresql_package_prefix} | \
@@ -83,6 +92,8 @@ sudo -u postgres -H \
      --locale=C \
      --pgdata=${data_dir} \
      --username=root
+echo "max_prepared_transactions = 1" | \
+  sudo -u postgres -H tee --append ${data_dir}/postgresql.conf
 sudo -u postgres -H \
      $(pg_config --bindir)/pg_ctl start \
      --pgdata=${data_dir}
@@ -92,11 +103,19 @@ cp -a \
    /host/expected \
    /tmp/
 cd /tmp
-if [ "$((${postgresql_version} < 13))" = 1 ]; then
+if [ ${postgresql_version} -lt 12 ]; then
+  rm sql/function/highlight-html/declarative-partitioning.sql
+  rm sql/function/wal-set-applied-position/declarative-partitioning.sql
+fi
+if [ ${postgresql_version} -lt 13 ]; then
   rm sql/full-text-search/text/single/declarative-partitioning.sql
 fi
 ruby /host/test/prepare.rb > schedule
-export PG_REGRESS_DIFF_OPTS="-u --color=always"
+PG_REGRESS_DIFF_OPTS="-u"
+if diff --help | grep -q color; then
+  PG_REGRESS_DIFF_OPTS="${PG_REGRESS_DIFF_OPTS} --color=always"
+fi
+export PG_REGRESS_DIFF_OPTS
 pg_regress=$(dirname $(pg_config --pgxs))/../test/regress/pg_regress
 
 echo "::endgroup::"
@@ -123,6 +142,28 @@ if [ ${pg_regress_status} -ne 0 ]; then
 fi
 
 
+echo "::group::Upgrade"
+
+sudo apt purge -V -y ${pgroonga_package}
+
+# Disable upgrade test for first time packages.
+# TODO: Automate this. We don't want to maintain this manually.
+# case ${postgresql_version} in
+#   16) # TODO: Remove this after 3.1.4 release.
+#     exit
+#     ;;
+# esac
+
+sudo apt install -V -y ${pgroonga_package}
+createdb upgrade
+psql upgrade -c 'CREATE EXTENSION pgroonga'
+apt install -V -y \
+  ${repositories_dir}/${os}/pool/${code_name}/*/*/*/*_${architecture}.deb
+psql upgrade -c 'ALTER EXTENSION pgroonga UPDATE'
+
+echo "::endgroup::"
+
+
 echo "::group::Postpare"
 
 sudo -u postgres -H \
